@@ -6,7 +6,7 @@
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/utils/mb/conversion_procs/cyrillic_and_mic/cyrillic_and_mic.c,v 1.9.4.4 2009/02/28 18:50:14 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/mb/conversion_procs/cyrillic_and_mic/cyrillic_and_mic.c,v 1.11.2.2 2008/03/20 10:38:34 heikki Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -23,20 +23,20 @@ PG_FUNCTION_INFO_V1(iso_to_mic);
 PG_FUNCTION_INFO_V1(mic_to_iso);
 PG_FUNCTION_INFO_V1(win1251_to_mic);
 PG_FUNCTION_INFO_V1(mic_to_win1251);
-PG_FUNCTION_INFO_V1(alt_to_mic);
-PG_FUNCTION_INFO_V1(mic_to_alt);
+PG_FUNCTION_INFO_V1(win866_to_mic);
+PG_FUNCTION_INFO_V1(mic_to_win866);
 PG_FUNCTION_INFO_V1(koi8r_to_win1251);
 PG_FUNCTION_INFO_V1(win1251_to_koi8r);
-PG_FUNCTION_INFO_V1(koi8r_to_alt);
-PG_FUNCTION_INFO_V1(alt_to_koi8r);
-PG_FUNCTION_INFO_V1(alt_to_win1251);
-PG_FUNCTION_INFO_V1(win1251_to_alt);
+PG_FUNCTION_INFO_V1(koi8r_to_win866);
+PG_FUNCTION_INFO_V1(win866_to_koi8r);
+PG_FUNCTION_INFO_V1(win866_to_win1251);
+PG_FUNCTION_INFO_V1(win1251_to_win866);
 PG_FUNCTION_INFO_V1(iso_to_koi8r);
 PG_FUNCTION_INFO_V1(koi8r_to_iso);
 PG_FUNCTION_INFO_V1(iso_to_win1251);
 PG_FUNCTION_INFO_V1(win1251_to_iso);
-PG_FUNCTION_INFO_V1(iso_to_alt);
-PG_FUNCTION_INFO_V1(alt_to_iso);
+PG_FUNCTION_INFO_V1(iso_to_win866);
+PG_FUNCTION_INFO_V1(win866_to_iso);
 
 extern Datum koi8r_to_mic(PG_FUNCTION_ARGS);
 extern Datum mic_to_koi8r(PG_FUNCTION_ARGS);
@@ -44,20 +44,20 @@ extern Datum iso_to_mic(PG_FUNCTION_ARGS);
 extern Datum mic_to_iso(PG_FUNCTION_ARGS);
 extern Datum win1251_to_mic(PG_FUNCTION_ARGS);
 extern Datum mic_to_win1251(PG_FUNCTION_ARGS);
-extern Datum alt_to_mic(PG_FUNCTION_ARGS);
-extern Datum mic_to_alt(PG_FUNCTION_ARGS);
+extern Datum win866_to_mic(PG_FUNCTION_ARGS);
+extern Datum mic_to_win866(PG_FUNCTION_ARGS);
 extern Datum koi8r_to_win1251(PG_FUNCTION_ARGS);
 extern Datum win1251_to_koi8r(PG_FUNCTION_ARGS);
-extern Datum koi8r_to_alt(PG_FUNCTION_ARGS);
-extern Datum alt_to_koi8r(PG_FUNCTION_ARGS);
-extern Datum alt_to_win1251(PG_FUNCTION_ARGS);
-extern Datum win1251_to_alt(PG_FUNCTION_ARGS);
+extern Datum koi8r_to_win866(PG_FUNCTION_ARGS);
+extern Datum win866_to_koi8r(PG_FUNCTION_ARGS);
+extern Datum win866_to_win1251(PG_FUNCTION_ARGS);
+extern Datum win1251_to_win866(PG_FUNCTION_ARGS);
 extern Datum iso_to_koi8r(PG_FUNCTION_ARGS);
 extern Datum koi8r_to_iso(PG_FUNCTION_ARGS);
 extern Datum iso_to_win1251(PG_FUNCTION_ARGS);
 extern Datum win1251_to_iso(PG_FUNCTION_ARGS);
-extern Datum iso_to_alt(PG_FUNCTION_ARGS);
-extern Datum alt_to_iso(PG_FUNCTION_ARGS);
+extern Datum iso_to_win866(PG_FUNCTION_ARGS);
+extern Datum win866_to_iso(PG_FUNCTION_ARGS);
 
 /* ----------
  * conv_proc(
@@ -76,17 +76,19 @@ static void iso2mic(const unsigned char *l, unsigned char *p, int len);
 static void mic2iso(const unsigned char *mic, unsigned char *p, int len);
 static void win12512mic(const unsigned char *l, unsigned char *p, int len);
 static void mic2win1251(const unsigned char *mic, unsigned char *p, int len);
-static void alt2mic(const unsigned char *l, unsigned char *p, int len);
-static void mic2alt(const unsigned char *mic, unsigned char *p, int len);
+static void win8662mic(const unsigned char *l, unsigned char *p, int len);
+static void mic2win866(const unsigned char *mic, unsigned char *p, int len);
 
 Datum
 koi8r_to_mic(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_KOI8R, PG_MULE_INTERNAL);
+	Assert(PG_GETARG_INT32(0) == PG_KOI8R);
+	Assert(PG_GETARG_INT32(1) == PG_MULE_INTERNAL);
+	Assert(len >= 0);
 
 	koi8r2mic(src, dest, len);
 
@@ -96,11 +98,13 @@ koi8r_to_mic(PG_FUNCTION_ARGS)
 Datum
 mic_to_koi8r(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_MULE_INTERNAL, PG_KOI8R);
+	Assert(PG_GETARG_INT32(0) == PG_MULE_INTERNAL);
+	Assert(PG_GETARG_INT32(1) == PG_KOI8R);
+	Assert(len >= 0);
 
 	mic2koi8r(src, dest, len);
 
@@ -110,11 +114,13 @@ mic_to_koi8r(PG_FUNCTION_ARGS)
 Datum
 iso_to_mic(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_ISO_8859_5, PG_MULE_INTERNAL);
+	Assert(PG_GETARG_INT32(0) == PG_ISO_8859_5);
+	Assert(PG_GETARG_INT32(1) == PG_MULE_INTERNAL);
+	Assert(len >= 0);
 
 	iso2mic(src, dest, len);
 
@@ -124,11 +130,13 @@ iso_to_mic(PG_FUNCTION_ARGS)
 Datum
 mic_to_iso(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_MULE_INTERNAL, PG_ISO_8859_5);
+	Assert(PG_GETARG_INT32(0) == PG_MULE_INTERNAL);
+	Assert(PG_GETARG_INT32(1) == PG_ISO_8859_5);
+	Assert(len >= 0);
 
 	mic2iso(src, dest, len);
 
@@ -138,11 +146,13 @@ mic_to_iso(PG_FUNCTION_ARGS)
 Datum
 win1251_to_mic(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_WIN1251, PG_MULE_INTERNAL);
+	Assert(PG_GETARG_INT32(0) == PG_WIN1251);
+	Assert(PG_GETARG_INT32(1) == PG_MULE_INTERNAL);
+	Assert(len >= 0);
 
 	win12512mic(src, dest, len);
 
@@ -152,11 +162,13 @@ win1251_to_mic(PG_FUNCTION_ARGS)
 Datum
 mic_to_win1251(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_MULE_INTERNAL, PG_WIN1251);
+	Assert(PG_GETARG_INT32(0) == PG_MULE_INTERNAL);
+	Assert(PG_GETARG_INT32(1) == PG_WIN1251);
+	Assert(len >= 0);
 
 	mic2win1251(src, dest, len);
 
@@ -164,29 +176,33 @@ mic_to_win1251(PG_FUNCTION_ARGS)
 }
 
 Datum
-alt_to_mic(PG_FUNCTION_ARGS)
+win866_to_mic(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_ALT, PG_MULE_INTERNAL);
+	Assert(PG_GETARG_INT32(0) == PG_WIN866);
+	Assert(PG_GETARG_INT32(1) == PG_MULE_INTERNAL);
+	Assert(len >= 0);
 
-	alt2mic(src, dest, len);
+	win8662mic(src, dest, len);
 
 	PG_RETURN_VOID();
 }
 
 Datum
-mic_to_alt(PG_FUNCTION_ARGS)
+mic_to_win866(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_MULE_INTERNAL, PG_ALT);
+	Assert(PG_GETARG_INT32(0) == PG_MULE_INTERNAL);
+	Assert(PG_GETARG_INT32(1) == PG_WIN866);
+	Assert(len >= 0);
 
-	mic2alt(src, dest, len);
+	mic2win866(src, dest, len);
 
 	PG_RETURN_VOID();
 }
@@ -194,16 +210,18 @@ mic_to_alt(PG_FUNCTION_ARGS)
 Datum
 koi8r_to_win1251(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_KOI8R, PG_WIN1251);
+	Assert(PG_GETARG_INT32(0) == PG_KOI8R);
+	Assert(PG_GETARG_INT32(1) == PG_WIN1251);
+	Assert(len >= 0);
 
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
+	buf = palloc(len * ENCODING_GROWTH_RATE);
 	koi8r2mic(src, buf, len);
-	mic2win1251(buf, dest, strlen(buf));
+	mic2win1251(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
@@ -212,66 +230,74 @@ koi8r_to_win1251(PG_FUNCTION_ARGS)
 Datum
 win1251_to_koi8r(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_WIN1251, PG_KOI8R);
+	Assert(PG_GETARG_INT32(0) == PG_WIN1251);
+	Assert(PG_GETARG_INT32(1) == PG_KOI8R);
+	Assert(len >= 0);
 
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
+	buf = palloc(len * ENCODING_GROWTH_RATE);
 	win12512mic(src, buf, len);
-	mic2koi8r(buf, dest, strlen(buf));
+	mic2koi8r(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
 }
 
 Datum
-koi8r_to_alt(PG_FUNCTION_ARGS)
+koi8r_to_win866(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_KOI8R, PG_ALT);
+	Assert(PG_GETARG_INT32(0) == PG_KOI8R);
+	Assert(PG_GETARG_INT32(1) == PG_WIN866);
+	Assert(len >= 0);
 
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
+	buf = palloc(len * ENCODING_GROWTH_RATE);
 	koi8r2mic(src, buf, len);
-	mic2alt(buf, dest, strlen(buf));
+	mic2win866(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
 }
 
 Datum
-alt_to_koi8r(PG_FUNCTION_ARGS)
+win866_to_koi8r(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_ALT, PG_KOI8R);
+	Assert(PG_GETARG_INT32(0) == PG_WIN866);
+	Assert(PG_GETARG_INT32(1) == PG_KOI8R);
+	Assert(len >= 0);
 
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
-	alt2mic(src, buf, len);
-	mic2koi8r(buf, dest, strlen(buf));
+	buf = palloc(len * ENCODING_GROWTH_RATE);
+	win8662mic(src, buf, len);
+	mic2koi8r(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
 }
 
 Datum
-alt_to_win1251(PG_FUNCTION_ARGS)
+win866_to_win1251(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_ALT, PG_WIN1251);
+	Assert(PG_GETARG_INT32(0) == PG_WIN866);
+	Assert(PG_GETARG_INT32(1) == PG_WIN1251);
+	Assert(len >= 0);
 
 	/*
 	 * Note: There are a few characters like the "Numero" sign that exist in
@@ -279,28 +305,30 @@ alt_to_win1251(PG_FUNCTION_ARGS)
 	 * not in KOI8R. As we use MULE_INTERNAL/KOI8R as an intermediary, we
 	 * will fail to convert those characters.
 	 */
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
-	alt2mic(src, buf, len);
-	mic2win1251(buf, dest, strlen(buf));
+	buf = palloc(len * ENCODING_GROWTH_RATE);
+	win8662mic(src, buf, len);
+	mic2win1251(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
 }
 
 Datum
-win1251_to_alt(PG_FUNCTION_ARGS)
+win1251_to_win866(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_WIN1251, PG_ALT);
+	Assert(PG_GETARG_INT32(0) == PG_WIN1251);
+	Assert(PG_GETARG_INT32(1) == PG_WIN866);
+	Assert(len >= 0);
 
-	/* Use mic/KOI8R as intermediary, see comment in alt_to_win1251() */
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
+	/* Use mic/KOI8R as intermediary, see comment in win866_to_win1251() */
+	buf = palloc(len * ENCODING_GROWTH_RATE);
 	win12512mic(src, buf, len);
-	mic2alt(buf, dest, strlen(buf));
+	mic2win866(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
@@ -309,16 +337,18 @@ win1251_to_alt(PG_FUNCTION_ARGS)
 Datum
 iso_to_koi8r(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_ISO_8859_5, PG_KOI8R);
+	Assert(PG_GETARG_INT32(0) == PG_ISO_8859_5);
+	Assert(PG_GETARG_INT32(1) == PG_KOI8R);
+	Assert(len >= 0);
 
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
+	buf = palloc(len * ENCODING_GROWTH_RATE);
 	iso2mic(src, buf, len);
-	mic2koi8r(buf, dest, strlen(buf));
+	mic2koi8r(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
@@ -327,16 +357,18 @@ iso_to_koi8r(PG_FUNCTION_ARGS)
 Datum
 koi8r_to_iso(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_KOI8R, PG_ISO_8859_5);
+	Assert(PG_GETARG_INT32(0) == PG_KOI8R);
+	Assert(PG_GETARG_INT32(1) == PG_ISO_8859_5);
+	Assert(len >= 0);
 
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
+	buf = palloc(len * ENCODING_GROWTH_RATE);
 	koi8r2mic(src, buf, len);
-	mic2iso(buf, dest, strlen(buf));
+	mic2iso(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
@@ -345,17 +377,19 @@ koi8r_to_iso(PG_FUNCTION_ARGS)
 Datum
 iso_to_win1251(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_ISO_8859_5, PG_WIN1251);
+	Assert(PG_GETARG_INT32(0) == PG_ISO_8859_5);
+	Assert(PG_GETARG_INT32(1) == PG_WIN1251);
+	Assert(len >= 0);
 
-	/* Use mic/KOI8R as intermediary, see comment in alt_to_win1251() */
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
+	/* Use mic/KOI8R as intermediary, see comment in win866_to_win1251() */
+	buf = palloc(len * ENCODING_GROWTH_RATE);
 	iso2mic(src, buf, len);
-	mic2win1251(buf, dest, strlen(buf));
+	mic2win1251(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
@@ -364,55 +398,61 @@ iso_to_win1251(PG_FUNCTION_ARGS)
 Datum
 win1251_to_iso(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_WIN1251, PG_ISO_8859_5);
+	Assert(PG_GETARG_INT32(0) == PG_WIN1251);
+	Assert(PG_GETARG_INT32(1) == PG_ISO_8859_5);
+	Assert(len >= 0);
 
-	/* Use mic/KOI8R as intermediary, see comment in alt_to_win1251() */
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
+	/* Use mic/KOI8R as intermediary, see comment in win866_to_win1251() */
+	buf = palloc(len * ENCODING_GROWTH_RATE);
 	win12512mic(src, buf, len);
-	mic2iso(buf, dest, strlen(buf));
+	mic2iso(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
 }
 
 Datum
-iso_to_alt(PG_FUNCTION_ARGS)
+iso_to_win866(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_ISO_8859_5, PG_ALT);
+	Assert(PG_GETARG_INT32(0) == PG_ISO_8859_5);
+	Assert(PG_GETARG_INT32(1) == PG_WIN866);
+	Assert(len >= 0);
 
-	/* Use mic/KOI8R as intermediary, see comment in alt_to_win1251() */
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
+	/* Use mic/KOI8R as intermediary, see comment in win866_to_win1251() */
+	buf = palloc(len * ENCODING_GROWTH_RATE);
 	iso2mic(src, buf, len);
-	mic2alt(buf, dest, strlen(buf));
+	mic2win866(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
 }
 
 Datum
-alt_to_iso(PG_FUNCTION_ARGS)
+win866_to_iso(PG_FUNCTION_ARGS)
 {
-	unsigned char *src = PG_GETARG_CSTRING(2);
-	unsigned char *dest = PG_GETARG_CSTRING(3);
+	unsigned char *src = (unsigned char *) PG_GETARG_CSTRING(2);
+	unsigned char *dest = (unsigned char *) PG_GETARG_CSTRING(3);
 	int			len = PG_GETARG_INT32(4);
 	unsigned char *buf;
 
-	CHECK_ENCODING_CONVERSION_ARGS(PG_ALT, PG_ISO_8859_5);
+	Assert(PG_GETARG_INT32(0) == PG_WIN866);
+	Assert(PG_GETARG_INT32(1) == PG_ISO_8859_5);
+	Assert(len >= 0);
 
-	/* Use mic/KOI8R as intermediary, see comment in alt_to_win1251() */
-	buf = palloc(len * ENCODING_GROWTH_RATE + 1);
-	alt2mic(src, buf, len);
-	mic2iso(buf, dest, strlen(buf));
+	/* Use mic/KOI8R as intermediary, see comment in win866_to_win1251() */
+	buf = palloc(len * ENCODING_GROWTH_RATE);
+	win8662mic(src, buf, len);
+	mic2iso(buf, dest, strlen((char *) buf));
 	pfree(buf);
 
 	PG_RETURN_VOID();
@@ -547,11 +587,11 @@ mic2win1251(const unsigned char *mic, unsigned char *p, int len)
 	mic2latin_with_table(mic, p, len, LC_KOI8_R, PG_WIN1251, koi2win);
 }
 
-/* alt2mic: CP866 to Mule internal code */
+/* win8662mic: CP866 to Mule internal code */
 static void
-alt2mic(const unsigned char *l, unsigned char *p, int len)
+win8662mic(const unsigned char *l, unsigned char *p, int len)
 {
-	static const unsigned char alt2koi[] = {
+	static const unsigned char win8662koi[] = {
 		0xe1, 0xe2, 0xf7, 0xe7, 0xe4, 0xe5, 0xf6, 0xfa,
 		0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef, 0xf0,
 		0xf2, 0xf3, 0xf4, 0xf5, 0xe6, 0xe8, 0xe3, 0xfe,
@@ -570,14 +610,14 @@ alt2mic(const unsigned char *l, unsigned char *p, int len)
 		0xb6, 0xa6, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
 	};
 
-	latin2mic_with_table(l, p, len, LC_KOI8_R, PG_ALT, alt2koi);
+	latin2mic_with_table(l, p, len, LC_KOI8_R, PG_WIN866, win8662koi);
 }
 
-/* mic2alt: Mule internal code to CP866 */
+/* mic2win866: Mule internal code to CP866 */
 static void
-mic2alt(const unsigned char *mic, unsigned char *p, int len)
+mic2win866(const unsigned char *mic, unsigned char *p, int len)
 {
-	static const unsigned char koi2alt[] = {
+	static const unsigned char koi2win866[] = {
 		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -596,5 +636,5 @@ mic2alt(const unsigned char *mic, unsigned char *p, int len)
 		0x9c, 0x9b, 0x87, 0x98, 0x9d, 0x99, 0x97, 0x9a
 	};
 
-	mic2latin_with_table(mic, p, len, LC_KOI8_R, PG_ALT, koi2alt);
+	mic2latin_with_table(mic, p, len, LC_KOI8_R, PG_WIN866, koi2win866);
 }
